/***************************************************************
From the book "Motif Programming, The Essentials...and More"
by Marshall Brain.
Published by Digital Press, ISBN 1-55558-089-0.
To order the book, call 1-800-DIGITAL and ask for EY-J816E-DP.

Copyright 1992, by Digital Equipment Corp.

This code demonstrates the use of a work proc to prevent
user interface stalling. Compare its behavior to 
"noworkproc.c". Adjust NUM_LINES here to match "noworkproc.c".
***************************************************************/

/* workproc.c */

#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>

#define NUM_LINES 25000
#define SIZE      500

XtAppContext context;
XmStringCharSet char_set=XmSTRING_DEFAULT_CHARSET;

GC gc;
Widget toplevel;
Widget drawing_area;
Widget button;
Widget form;
int rand_seed=10;
int line_index;

void setup_gc()
/* set up the graphics context. */
{
    int foreground,background;
    XGCValues vals;
    Arg al[10];
    int ac;

    /* get the current fg and bg colors. */
    ac=0;
    XtSetArg(al[ac],XmNforeground,&foreground); ac++;
    XtSetArg(al[ac],XmNbackground,&background); ac++;
    XtGetValues(drawing_area,al,ac);

    /* create the gc. */
    vals.foreground = foreground;
    vals.background = background;
    gc=XtGetGC(drawing_area,GCForeground | GCBackground,&vals);
}

int rand()
/* from K&R */
{
    rand_seed = rand_seed * 1103515245 +12345;
    return (unsigned int)(rand_seed / 65536) % 32768;
}

Boolean work_proc(client_data)
    XtPointer client_data; 
/* The work proc divides the drawing of lines up into groups of 1000
   lines so that the event loop gets returned to regularly. */
{
    int x;

    for (x=0; x<=NUM_LINES/25 && line_index<NUM_LINES; x++)
    {
        XDrawLine(XtDisplay(drawing_area),XtWindow(drawing_area), 
            gc, rand()%SIZE,rand()%SIZE,rand()%SIZE,rand()%SIZE);
        line_index++;
    }
    if (line_index<NUM_LINES)
        return False;
    else
        return True;
}

void exposeCB(w,client_data,call_data)
    Widget w;
    XtPointer client_data; 
    XtPointer call_data;
/* called whenever drawing area is exposed. */
{
    XtWorkProcId id;

    line_index=0;
    id=XtAppAddWorkProc(context,work_proc,NULL);
}

void buttonCB(w,client_data,call_data)
    Widget w;
    XtPointer client_data; 
    XtPointer call_data;
/* called whenever quit button is clicked. */
{
    printf("Quit button clicked\n");
    exit(0);
}

main(argc,argv)
    int argc;
    char *argv[];
{
    Arg al[10];
    int ac;

    /* create the toplevel shell */
    toplevel = XtAppInitialize(&context,"",NULL,0,&argc,argv,
        NULL,NULL,0);

    /* set window size. */
    ac=0;
    XtSetArg(al[ac],XmNheight,SIZE); ac++;
    XtSetArg(al[ac],XmNwidth,SIZE); ac++;
    XtSetValues(toplevel,al,ac);

    /* create a form to hold widgets */
    ac=0;
    form=XmCreateForm(toplevel,"form",al,ac);
    XtManageChild(form);    

    /* create a push button */
    ac=0;
    XtSetArg(al[ac],XmNlabelString,
        XmStringCreate("Quit",char_set)); ac++;
    XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
    XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
    XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
    button=XmCreatePushButton(form,"button",al,ac);
    XtManageChild(button);
    XtAddCallback(button,XmNactivateCallback,buttonCB,NULL);

    /* create a drawing area widget. */
    ac=0;
    XtSetArg(al[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
    XtSetArg(al[ac], XmNtopWidget, button); ac++;
    XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
    XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
    XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
    drawing_area=XmCreateDrawingArea(form,"drawing_area",al,ac);
    XtManageChild(drawing_area);
    XtAddCallback(drawing_area,XmNexposeCallback,exposeCB,NULL);

    setup_gc();

    XtRealizeWidget(toplevel);
    XtAppMainLoop(context);
}
